@@ -12,3 +12,5 @@
 #undef LPR_PATH
 #undef HAVE_GSTROKE
 #undef ALPHA_PARAMS
+#undef HAVE_XINERAMA
+#undef BONOBO_DEBUG
