@@ -33,12 +33,14 @@ struct _GgvControlPrivate {
 
 	GtkWidget *root;
 
-	BonoboUIComponent *uic;
+	BonoboUIComponent *uic, *popup_uic;
 };
 
 struct _GgvControlClassPrivate {
 #ifdef DYNAMIC_ZOOM_MENUS
 	BonoboUINode *zoom_level_menus;
+#else
+	int dummy;
 #endif /* DYNAMIC_ZOOM_MENUS */
 };
 
@@ -284,6 +286,18 @@ listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
 	}
 }
 
+static gboolean
+ggv_control_button_press_event(GtkWidget *widget, GdkEventButton *event,
+							   gpointer data)
+{
+	GgvControl *control = GGV_CONTROL(data);
+
+	if(event->button == 3) {
+		return bonobo_control_do_popup(BONOBO_CONTROL(control), 3, event->time);
+	}
+	return FALSE;
+}
+
 static gboolean
 ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 							gpointer data)
@@ -384,21 +398,6 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	return TRUE;
 }
 
-static void
-verb_ZoomLevel_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
-{
-	GgvControl *control;
-
-	g_return_if_fail (user_data != NULL);
-	g_return_if_fail (GGV_IS_CONTROL (user_data));
-
-	control = GGV_CONTROL (user_data);
-
-	g_signal_emit_by_name (G_OBJECT (control->priv->zoomable),
-						   "zoom_in");
-}
-
-
 static void
 verb_ZoomIn_cb (BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
@@ -458,30 +457,60 @@ verb_ZoomToFit_cb (BonoboUIComponent *uic, gpointer user_data,
 }
 
 static void
-set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
+verb_ZoomToFitWidth_cb (BonoboUIComponent *uic, gpointer user_data,
+						const char *cname)
+{
+	GgvControl *control;
+	gfloat zoom_level;
+
+	g_return_if_fail (user_data != NULL);
+	g_return_if_fail (GGV_IS_CONTROL (user_data));
+
+	control = GGV_CONTROL (user_data);
+
+	zoom_level = ggv_postscript_view_zoom_to_fit(control->priv->ps_view, TRUE);
+
+	g_signal_emit_by_name(G_OBJECT(control->priv->zoomable), "set_zoom_level",
+						  zoom_level);
+
+}
+
+static void
+_set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolean sens)
 {
 	gchar *prop_val = sens?"1":"0";
 
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomIn",
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomIn",
 								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomOut",
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomOut",
 								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomToDefault",
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToDefault",
 								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/ZoomToFit",
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFit",
 								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/menu/ViewPlaceholder/View/ZoomMenu/Zoom",
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFitWidth",
 								 "sensitive", prop_val, NULL);
 }
 
+static void
+set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	_set_zoom_items_sensitivity(control, control->priv->uic, sens);
+	if(control->priv->popup_uic != NULL)
+		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
+}
+
 static BonoboUIVerb ggv_control_verbs[] = {
-		BONOBO_UI_VERB ("ZoomIn",        verb_ZoomIn_cb),
-		BONOBO_UI_VERB ("ZoomOut",       verb_ZoomOut_cb),
-		BONOBO_UI_VERB ("ZoomToDefault", verb_ZoomToDefault_cb),
-		BONOBO_UI_VERB ("ZoomToFit",     verb_ZoomToFit_cb),
+		BONOBO_UI_VERB ("ZoomIn",         verb_ZoomIn_cb),
+		BONOBO_UI_VERB ("ZoomOut",        verb_ZoomOut_cb),
+		BONOBO_UI_VERB ("ZoomToDefault",  verb_ZoomToDefault_cb),
+		BONOBO_UI_VERB ("ZoomToFit",      verb_ZoomToFit_cb),
+		BONOBO_UI_VERB ("ZoomToFitWidth", verb_ZoomToFitWidth_cb),
 		BONOBO_UI_VERB_END
 };
 
+extern BonoboUIVerb ggv_postscript_view_verbs[];
+
 static void
 ggv_control_create_ui (GgvControl *control)
 {
@@ -496,9 +525,12 @@ ggv_control_create_ui (GgvControl *control)
 
 	klass = GGV_CONTROL_CLASS(G_OBJECT_GET_CLASS(control));
 
-	bonobo_ui_util_set_ui(control->priv->uic, NULL,
-						  "ggv-control-ui.xml", "GGV", NULL);
-
+	bonobo_ui_util_set_ui(control->priv->uic, NULL, "ggv-control-ui.xml",
+						  "GGV", NULL);
+	control->priv->popup_uic = bonobo_control_get_popup_ui_component(BONOBO_CONTROL(control));
+	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-postscript-view-ui.xml", "GGV", NULL);
+	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-control-ui.xml", "GGV", NULL);
+	
 #ifdef DYNAMIC_ZOOM_MENUS
 	zoom_level_menus = bonobo_ui_node_copy(klass->priv->zoom_level_menus,
 										   TRUE);
@@ -506,20 +538,28 @@ ggv_control_create_ui (GgvControl *control)
 								 "/menu/ViewPlaceholder/View/ZoomMenu/Zoom/ZoomLevels",
 								 zoom_level_menus, NULL);
 #else
-	for(i = 0; zoom_level_items[i].path; i++)
+	for(i = 0; zoom_level_items[i].path; i++) {
 		bonobo_ui_component_add_listener(control->priv->uic,
 										 zoom_level_items[i].path + strlen("/commands/"),
 										 listener_ZoomLevel_cb, control);
-	
+	}
 #endif /* DYNAMIC_ZOOM_MENUS */
 
 	set_zoom_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
 
 	sync_zoom_level_items(control);
 
-	bonobo_ui_component_add_verb_list_with_data
-		(control->priv->uic, ggv_control_verbs,
-		 control);
+	bonobo_ui_component_add_verb_list_with_data(control->priv->uic,
+												ggv_control_verbs,
+												control);
+	bonobo_ui_component_add_verb_list_with_data(control->priv->popup_uic,
+												ggv_control_verbs,
+												control);
+	bonobo_ui_component_add_verb_list_with_data(control->priv->popup_uic,
+												ggv_postscript_view_verbs,
+												control->priv->ps_view);
+	ggv_postscript_view_set_popup_ui_component(control->priv->ps_view,
+											   control->priv->popup_uic);
 }
 
 static void
@@ -657,6 +697,8 @@ ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 	view = ggv_postscript_view_get_widget (control->priv->ps_view);
 	g_signal_connect(G_OBJECT(view), "key_press_event",
 					 G_CALLBACK(ggv_control_key_press_event), control);
+	g_signal_connect(G_OBJECT(view), "button_press_event",
+					 G_CALLBACK(ggv_control_button_press_event), control);
 
 	control->priv->root = gtk_scrolled_window_new(ggv_postscript_view_get_hadj(ps_view),
 												  ggv_postscript_view_get_vadj(ps_view));
@@ -733,7 +775,7 @@ ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 
 	bonobo_control_set_properties (BONOBO_CONTROL(control),
 								   BONOBO_OBJREF(property_bag), NULL);
-	mask =  "Bonobo/Property:change:page";
+	mask = "Bonobo/Property:change:page";
 	bonobo_event_source_client_add_listener(BONOBO_OBJREF(property_bag),
 											(BonoboListenerCallbackFn)ps_view_property_changed_handler,
 											mask, NULL, control);
