@@ -23,6 +23,7 @@
 #include <ps.h>
 #include <gsdefaults.h>
 #include <ggvutils.h>
+#include <ggv-msg-window.h>
 
 #include <bonobo.h>
 
@@ -40,10 +41,12 @@ struct _GgvPostScriptViewPrivate {
 	gdouble prev_x, prev_y;
 	gchar *tmp_name;
 
+	GgvMsgWindow *msg_win;
+
 	BonoboPropertyBag     *property_bag;
 	BonoboPropertyControl *property_control;
 
-	BonoboUIComponent     *uic;
+	BonoboUIComponent     *uic, *popup_uic;
 
 	gboolean               zoom_fit;
 };
@@ -52,6 +55,8 @@ struct _GgvPostScriptViewClassPrivate {
 #ifdef DYNAMIC_PAPER_MENUS
 	BonoboUINode   *size_menus;
 	gchar          **size_paths;
+#else
+	int dummy;
 #endif /* DYNAMIC_PAPER_MENUS */
 };
 
@@ -144,7 +149,7 @@ get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 }
 
 static void
-set_page_item_sensitivity(GgvPostScriptView *ps_view)
+_set_page_item_sensitivity(GgvPostScriptView *ps_view, BonoboUIComponent *uic)
 {
 	GtkGS *gs = GTK_GS(ps_view->priv->gs);
 	gchar *prop_val;
@@ -152,22 +157,28 @@ set_page_item_sensitivity(GgvPostScriptView *ps_view)
 	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
 				gtk_gs_get_current_page(gs) < 0)?
 		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/NextPage",
+	bonobo_ui_component_set_prop(uic, "/commands/NextPage",
 								 "sensitive", prop_val, NULL);
 	prop_val = (gtk_gs_get_current_page(gs) <= 0 || !gs->structured_doc)?
 		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/PrevPage",
+	bonobo_ui_component_set_prop(uic, "/commands/PrevPage",
 								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FirstPage",
+	bonobo_ui_component_set_prop(uic, "/commands/FirstPage",
 								 "sensitive", prop_val, NULL);
 	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
 				!gs->structured_doc || gtk_gs_get_current_page(gs) < 0)?
 		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/LastPage",
+	bonobo_ui_component_set_prop(uic, "/commands/LastPage",
 								 "sensitive", prop_val, NULL);
-
 }
 
+static void
+set_page_item_sensitivity(GgvPostScriptView *ps_view)
+{
+	_set_page_item_sensitivity(ps_view, ps_view->priv->uic);
+	if(ps_view->priv->popup_uic != NULL)
+		_set_page_item_sensitivity(ps_view, ps_view->priv->popup_uic);
+}
 
 static void
 notify_page_count_change(GgvPostScriptView *ps_view)
@@ -247,7 +258,6 @@ ggv_postscript_view_get_page_count(GgvPostScriptView *ps_view)
 	return gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
 }
 
-
 static void
 load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 {
@@ -269,7 +279,7 @@ load_ps_from_stream (BonoboPersistStream *ps,
 					 CORBA_Environment *ev)
 {
 	GgvPostScriptView *ps_view;
-	guint8 *buffer;
+	Bonobo_Stream_iobuf *buffer;
 	CORBA_long len_read;
 	FILE *tmpfile;
 	int fd;
@@ -296,27 +306,36 @@ load_ps_from_stream (BonoboPersistStream *ps,
 		close(fd);
 		return;
 	}
+
 	do {
-		buffer = bonobo_stream_client_read(stream, 32768, &len_read, ev);
-		if(buffer) {
-			if(len_read > 0) {
-				if(fwrite(buffer, 1, len_read, tmpfile) != len_read) {
-					g_free(buffer);
-					fclose(tmpfile);
-					return;
-				}
+		Bonobo_Stream_read (stream, 32768, &buffer, ev);
+		if (ev->_major != CORBA_NO_EXCEPTION)
+			goto exit_clean;
+
+		len_read = buffer->_length;
+
+		if (buffer->_buffer && len_read)
+			if(fwrite(buffer->_buffer, 1, len_read, tmpfile) != len_read) {
+				CORBA_free (buffer);
+				goto exit_clean;
 			}
-			g_free(buffer);
-		}
-	} while(len_read != 32768);
+
+		CORBA_free (buffer);
+	} while (len_read > 0);
+
 	fclose(tmpfile);
 	load_ps(ps_view, ps_view->priv->tmp_name);
+	return;
+
+ exit_clean:
+	fclose (tmpfile);
+	return;
 }
 
 /* 
  * handlers for mouse actions
  */
-static void
+static gboolean
 view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GgvPostScriptView *ps_view;
@@ -346,10 +365,14 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 						 pan_cursor, GDK_CURRENT_TIME);
 		ps_view->priv->prev_x = wx;
 		ps_view->priv->prev_y = wy;
+
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
-static void
+static gboolean
 view_button_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 {
 	GgvPostScriptView *ps_view;
@@ -364,10 +387,14 @@ view_button_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 		gdk_pointer_ungrab(GDK_CURRENT_TIME);
 		gtk_grab_remove(widget);
 		gtk_gs_end_scroll(GTK_GS(ps_view->priv->gs));
+
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
-static void
+static gboolean
 view_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 {
 	GgvPostScriptView *ps_view;
@@ -383,7 +410,11 @@ view_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 					  -(gint)event->y + ps_view->priv->prev_y);
 		ps_view->priv->prev_x = (gint)event->x;
 		ps_view->priv->prev_y = (gint)event->y;
+
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
 
@@ -641,17 +672,20 @@ sync_size_items(GgvPostScriptView *ps_view)
 	}
 }
 
+BonoboUIVerb ggv_postscript_view_verbs[] = {
+	BONOBO_UI_VERB ("NextPage", verb_NextPage_cb),
+	BONOBO_UI_VERB ("PrevPage", verb_PrevPage_cb),
+	BONOBO_UI_VERB ("LastPage", verb_LastPage_cb),
+	BONOBO_UI_VERB ("FirstPage", verb_FirstPage_cb),
+	BONOBO_UI_VERB_END
+};
+
 static void
 ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 {
-	static BonoboUIVerb ggv_postscript_view_verbs[] = {
-		BONOBO_UI_VERB ("NextPage", verb_NextPage_cb),
-		BONOBO_UI_VERB ("PrevPage", verb_PrevPage_cb),
-		BONOBO_UI_VERB ("LastPage", verb_LastPage_cb),
-		BONOBO_UI_VERB ("FirstPage", verb_FirstPage_cb),
-		BONOBO_UI_VERB_END
-	};
+#ifdef DYNAMIC_PAPER_MENUS
 	BonoboUINode *new_size_menus;
+#endif /* DYNAMIC_PAPER_MENUS */
 	BonoboUINode *ui_root;
 	GgvPostScriptViewClass *klass = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view));
 	gint i;
@@ -663,6 +697,9 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 	bonobo_ui_util_set_ui(ps_view->priv->uic, DATADIR,
 						  "ggv-postscript-view-ui.xml", 
 						  "GGV", NULL);
+	bonobo_ui_component_add_verb_list_with_data(ps_view->priv->uic,
+												ggv_postscript_view_verbs,
+												ps_view);
 
 #ifdef DYNAMIC_PAPER_MENUS
 	new_size_menus = bonobo_ui_node_copy(klass->priv->size_menus, TRUE);
@@ -692,10 +729,6 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 	bonobo_ui_component_add_listener(ps_view->priv->uic, "OverrideOrientation",
 									 listener_Toggle_cb, ps_view);
 
-	bonobo_ui_component_add_verb_list_with_data(ps_view->priv->uic,
-												ggv_postscript_view_verbs,
-												ps_view);
-
 	set_page_item_sensitivity(ps_view);
 	sync_orientation_items(ps_view);
 	sync_size_items(ps_view);
@@ -1235,21 +1268,30 @@ ggv_postscript_view_destroy(BonoboObject *object)
 
 	ps_view = GGV_POSTSCRIPT_VIEW(object);
 
-	if(ps_view->priv->property_bag)
+	if(ps_view->priv->property_bag) {
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->property_bag));
-	ps_view->priv->property_bag = NULL;
+		ps_view->priv->property_bag = NULL;
+	}
 
-	if(ps_view->priv->property_control)
+	if(ps_view->priv->property_control) {
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->property_control));
-	ps_view->priv->property_control = NULL;
+		ps_view->priv->property_control = NULL;
+	}
 
-	if(ps_view->priv->gs)
+	if(ps_view->priv->gs) {
 		gtk_widget_destroy(ps_view->priv->gs);
-	ps_view->priv->gs = NULL;
+		ps_view->priv->gs = NULL;
+	}
 
-	if(ps_view->priv->uic)
+	if(ps_view->priv->msg_win) {
+		ggv_msg_window_free(ps_view->priv->msg_win);
+		ps_view->priv->msg_win = NULL;
+	}
+
+	if(ps_view->priv->uic) {
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->uic));
-	ps_view->priv->uic = NULL;
+		ps_view->priv->uic = NULL;
+	}
 
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
@@ -1430,13 +1472,18 @@ property_control_get_cb(BonoboPropertyControl *property_control,
 #endif
 }
 
-#ifdef BONOBO_DEBUG
 static void
 interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
 {
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+
+	if(ps_view->priv->msg_win)
+		ggv_msg_window_add_text(ps_view->priv->msg_win, msg, TRUE);
+
+#ifdef BONOBO_DEBUG
 	g_warning(msg);
-}
 #endif /* BONOBO_DEBUG */
+}
 
 static void
 interpreter_error_cb(GtkGS *gs, gint status, gpointer data)
@@ -1460,6 +1507,8 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 	if(!gconf_is_initialized())
 		gconf_init(0, NULL, NULL);
 	
+	ps_view->priv->msg_win = ggv_msg_window_new(_("GhostScript output"));
+
 	ps_view->priv->gs = GTK_WIDGET(gs);
 	g_object_set(gs, "can-focus", TRUE, NULL);
 	gtk_widget_set_events(GTK_WIDGET(gs), 
@@ -1471,10 +1520,8 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 					 G_CALLBACK(view_button_release_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "motion_notify_event",
 					 G_CALLBACK(view_motion_cb), ps_view);
-#ifdef BONOBO_DEBUG
 	g_signal_connect(G_OBJECT(gs), "interpreter_message",
-					 G_CALLBACK(interpreter_message_cb), NULL);
-#endif /* BONOBO_DEBUG */
+					 G_CALLBACK(interpreter_message_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "interpreter_error",
 					 G_CALLBACK(interpreter_error_cb), ps_view);
 
@@ -1584,3 +1631,11 @@ ggv_postscript_view_get_vadj(GgvPostScriptView *ps_view)
 {
 	return GTK_GS(ps_view->priv->gs)->vadj;
 }
+
+void
+ggv_postscript_view_set_popup_ui_component(GgvPostScriptView *ps_view,
+										   BonoboUIComponent *uic)
+{
+	ps_view->priv->popup_uic = uic;
+	_set_page_item_sensitivity(ps_view, uic);
+}
