@@ -59,7 +59,8 @@ void                  ggv_postscript_view_unset_ui_container    (GgvPostScriptVi
 GtkWidget             *ggv_postscript_view_get_widget           (GgvPostScriptView *image_view);
 void	              ggv_postscript_view_print                 (GgvPostScriptView *image_view,
 								 gboolean preview);
-
+void                  ggv_postscript_view_set_popup_ui_component(GgvPostScriptView *ps_view,
+								 BonoboUIComponent *uic);
 
 /* Zooming */
 float ggv_postscript_view_get_zoom_factor (GgvPostScriptView *image_view);
