@@ -139,7 +139,7 @@ ggv_prefs_dialog_delete_event(GtkWidget *widget, GdkEventAny *event, gpointer da
 {
         gtk_widget_hide(widget);
 
-        return FALSE;
+        return TRUE;
 }
 
 static void
@@ -147,7 +147,7 @@ ggv_prefs_dialog_class_init(GgvPrefsDialogClass *klass)
 {
 	parent_class = g_type_class_peek_parent (klass);
 
-        GTK_WIDGET_CLASS(parent_class)->delete_event = ggv_prefs_dialog_delete_event;
+        GTK_WIDGET_CLASS(klass)->delete_event = ggv_prefs_dialog_delete_event;
 }
 
 static void
@@ -914,6 +914,7 @@ ggv_prefs_changed(GConfClient* client, guint cnxn_id,
                 ggv_toolbar_labels = gconf_client_get_bool(gconf_client,
                                                            "/apps/ggv/layout/toolbarlabels",
                                                            NULL);
+                ggv_window_apply_prefs(NULL);
         }
         else if(!strcmp(entry->key, "/apps/ggv/layout/showmenubar")) {
                 ggv_menubar = gconf_client_get_bool(gconf_client,
