@@ -35,6 +35,96 @@ enum {
         TARGET_URI_LIST,
 };
 
+static void
+ggv_window_drag_data_received(GtkWidget *widget,
+                              GdkDragContext *context,
+                              gint x, gint y,
+                              GtkSelectionData *selection_data,
+                              guint info, guint time)
+{
+        GgvWindow *win = GGV_WINDOW(widget);
+
+	if (info != TARGET_URI_LIST)
+		return;
+
+        win->uris_to_open = g_strsplit(selection_data->data, "\r\n", 0);
+        if (context->suggested_action == GDK_ACTION_ASK) {
+                GtkWidget *menu = gtk_menu_new ();
+		
+                bonobo_window_add_popup (BONOBO_WINDOW (win), 
+                                         GTK_MENU (menu), 
+                                         "/popups/DnD");
+                gtk_menu_popup (GTK_MENU (menu),
+                                NULL,
+                                NULL,
+                                NULL,
+                                NULL,
+                                0,
+                                GDK_CURRENT_TIME);
+        }
+        else {
+                GtkWidget *newwin;
+                gchar **uri = win->uris_to_open;
+
+                if(win->current_page < 0)
+                        newwin = GTK_WIDGET(win);
+                else
+                        newwin = NULL;
+                while(*uri) {
+                        if(!g_strncasecmp("file:", *uri, 5)) {
+                                if(newwin == NULL) {
+                                        newwin = ggv_window_new();
+                                        gtk_widget_show(newwin);
+                                }
+                                ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
+                                newwin = NULL;
+                        }
+                        uri++;
+                }
+                g_strfreev(win->uris_to_open);
+                win->uris_to_open = NULL;
+        }
+}
+
+static void
+sync_settings_menu_items(GgvWindow *win)
+{
+        if(win->uic == NULL)
+                return;
+
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowMenus", "state",
+                                     win->show_menus?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowToolbar", "state",
+                                     win->show_toolbar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowSidebar", "state",
+                                     win->show_sidebar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->uic, "/commands/SettingsShowStatusbar", "state",
+                                     win->show_statusbar?"1":"0", NULL);
+}
+
+static void
+sync_settings_popup_items(GgvWindow *win)
+{
+        if(win->popup_uic == NULL)
+                return;
+
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowMenus", "state",
+                                     win->show_menus?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowToolbar", "state",
+                                     win->show_toolbar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowSidebar", "state",
+                                     win->show_sidebar?"1":"0", NULL);
+        bonobo_ui_component_set_prop(win->popup_uic, "/commands/SettingsShowStatusbar", "state",
+                                     win->show_statusbar?"1":"0", NULL);
+}
+
+static void
+sync_toolbar_style(GgvWindow *win)
+{
+        bonobo_ui_component_set_prop(win->uic, "/Toolbar", "look",
+                                     ggv_toolbar_labels?"both":"icon", NULL);
+}
+        
 static CORBA_string
 ggv_window_get_ps_from_control(GgvWindow *win, gint *active_rows)
 {
@@ -154,7 +244,7 @@ control_property_changed_handler(BonoboListener    *listener,
                         return;
 
                 win->current_page = BONOBO_ARG_GET_INT(any);
-                set_file_items_sensitivity(win, win->current_page != -1);
+                set_file_items_sensitivity(win, win->current_page >= 0);
                 if(win->current_page < 0) {
                         GtkWidget *dlg;
                         dlg = gtk_message_dialog_new(GTK_WINDOW(win),
@@ -399,7 +489,6 @@ static void
 verb_FileNew(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GtkWidget *win = ggv_window_new();
-
         gtk_widget_show(win);
 }
 
@@ -449,7 +538,7 @@ listener_SettingsShow (BonoboUIComponent *uic, const char *path,
         if(!strcmp(path, "SettingsShowMenus")) {
                 if(window->show_menus != state_f) {
                         window->show_menus = state_f;
-                        bonobo_ui_component_set_prop(uic, "/menu", "hidden",
+                        bonobo_ui_component_set_prop(window->uic, "/menu", "hidden",
                                                      state_f?"0":"1", NULL);
                 }
         }
@@ -465,7 +554,7 @@ listener_SettingsShow (BonoboUIComponent *uic, const char *path,
 	else if(!strcmp(path, "SettingsShowToolbar")) {
                 if(window->show_toolbar != state_f) {
                         window->show_toolbar = state_f;
-                        bonobo_ui_component_set_prop(uic, "/Toolbar", "hidden",
+                        bonobo_ui_component_set_prop(window->uic, "/Toolbar", "hidden",
                                                      state_f?"0":"1", NULL);
                 }
         }
@@ -478,6 +567,10 @@ listener_SettingsShow (BonoboUIComponent *uic, const char *path,
                                 gtk_widget_hide(window->statusbar);
                 }
         }
+        if(uic == window->uic)
+                sync_settings_popup_items(window);
+        else
+                sync_settings_menu_items(window);
 }
 
 static void
@@ -523,6 +616,58 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 	ggv_raise_and_focus_widget (about);
 }
 
+static void
+verb_DnDNewWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+        GgvWindow *win = GGV_WINDOW(user_data);
+        gchar **uri;
+
+        uri = win->uris_to_open;
+        while(*uri) {
+                GtkWidget *newwin;
+                if(!g_strncasecmp("file:", *uri, 5)) {
+                        newwin = ggv_window_new();
+                        gtk_widget_show(newwin);
+                        ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
+                }
+                uri++;
+        }
+        g_strfreev(win->uris_to_open);
+        win->uris_to_open = NULL;
+}
+
+static void
+verb_DnDThisWindow(BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+        GgvWindow *win = GGV_WINDOW(user_data);
+        GtkWidget *newwin;
+        gchar **uri;
+
+        uri = win->uris_to_open;
+        newwin = GTK_WIDGET(win);
+        while(*uri) {
+                if(!g_strncasecmp("file:", *uri, 5)) {
+                        if(newwin == NULL) {
+                                newwin = ggv_window_new();
+                                gtk_widget_show(newwin);
+                        }
+                        ggv_window_load(GGV_WINDOW(newwin), (*uri) + 5);
+                        newwin = NULL;
+                }
+                uri++;
+        }
+        g_strfreev(win->uris_to_open);
+        win->uris_to_open = NULL;	
+}
+
+static void
+verb_DnDCancel(BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+        GgvWindow *win = GGV_WINDOW(user_data);
+        g_strfreev(win->uris_to_open);
+        win->uris_to_open = NULL;	
+}
+
 /* our verb list */
 static BonoboUIVerb ggv_app_verbs[] = {
         BONOBO_UI_VERB("FileNew", verb_FileNew),
@@ -534,6 +679,9 @@ static BonoboUIVerb ggv_app_verbs[] = {
         BONOBO_UI_VERB("FileExit", verb_FileExit),
         BONOBO_UI_VERB("SettingsPreferences", verb_SettingsPreferences),
         BONOBO_UI_VERB("HelpAbout", verb_HelpAbout),
+        BONOBO_UI_VERB("DnDNewWindow", verb_DnDNewWindow),
+        BONOBO_UI_VERB("DnDThisWindow", verb_DnDThisWindow),
+        BONOBO_UI_VERB("DnDCancel", verb_DnDCancel),
         BONOBO_UI_VERB_END
 };
 
@@ -546,9 +694,8 @@ control_frame_activate_uri(BonoboControlFrame *control_frame, const char *uri,
 
 	g_return_if_fail(uri != NULL);
 
-	win = GGV_WINDOW(ggv_window_new());
-	
 	if(g_strncasecmp("file:", uri, 5) == 0) {
+                win = GGV_WINDOW(ggv_window_new());
                 ggv_window_load(win, uri + 5);
                 gtk_widget_show(GTK_WIDGET(win));
         }
@@ -706,6 +853,7 @@ static gboolean
 ggv_window_delete_event(GtkWidget *widget, GdkEventAny *e)
 {
         ggv_window_close(GGV_WINDOW(widget));
+
         return TRUE;
 }
 
@@ -725,10 +873,7 @@ ggv_window_class_init(GgvWindowClass *class)
 	object_class->destroy = ggv_window_destroy;
 
 	widget_class->delete_event = ggv_window_delete_event;
-#if 0
-	widget_class->key_press_event = ggv_window_key_press_event;
 	widget_class->drag_data_received = ggv_window_drag_data_received;
-#endif
 }
 
 static void
@@ -769,7 +914,7 @@ ggv_window_new(void)
 {
         GgvWindow *win;
 	BonoboUIContainer *ui_container;
-	BonoboUIComponent *uic;
+	BonoboUIComponent *uic, *popup_uic;
 	GdkGeometry geometry;
         CORBA_Environment ev;
         Bonobo_Control control;
@@ -778,6 +923,10 @@ ggv_window_new(void)
         gchar *mask;
         GtkTreeSelection *sel;
 
+	static const GtkTargetEntry drag_types[] = {
+		{ "text/uri-list", 0, TARGET_URI_LIST }
+	};
+
         /* get the control */
         CORBA_exception_init(&ev);
         control = bonobo_get_object(GGV_POSTSCRIPT_VIEWER_CONTROL_IID,
@@ -791,7 +940,7 @@ ggv_window_new(void)
 	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, NULL));
         win->control = control;
 
-        win->current_page = 0;
+        win->current_page = -2;
         win->show_toolbar = ggv_toolbar;
         win->show_menus = ggv_menubar;
         win->show_statusbar = ggv_statusbar;
@@ -884,12 +1033,19 @@ ggv_window_new(void)
                                 "ggv", _("Gnome Ghostview"));
         bonobo_window_set_contents(BONOBO_WINDOW(win), win->vbox);
 
+	gtk_drag_dest_set (GTK_WIDGET (win),
+			   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+			   drag_types,
+			   sizeof (drag_types) / sizeof (drag_types[0]),
+			   GDK_ACTION_COPY | GDK_ACTION_ASK);
+
 	/* add menu and toolbar */
 	uic = bonobo_ui_component_new("ggv");
         win->uic = uic;
 	bonobo_ui_component_set_container(uic, BONOBO_OBJREF(ui_container),
                                           NULL);
 	bonobo_ui_util_set_ui(uic, NULL, "ggv-ui.xml", "GGV", NULL);
+        sync_toolbar_style(win);
 	bonobo_ui_component_add_verb_list_with_data(uic, ggv_app_verbs, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowMenus",
                                          listener_SettingsShow, win);
@@ -899,14 +1055,8 @@ ggv_window_new(void)
                                          listener_SettingsShow, win);
 	bonobo_ui_component_add_listener(uic, "SettingsShowStatusbar",
                                          listener_SettingsShow, win);
-        bonobo_ui_component_set_prop(uic, "/commands/SettingsShowMenus", "state",
-                                     win->show_menus?"1":"0", NULL);
-        bonobo_ui_component_set_prop(uic, "/commands/SettingsShowToolbar", "state",
-                                     win->show_toolbar?"1":"0", NULL);
-        bonobo_ui_component_set_prop(uic, "/commands/SettingsShowSidebar", "state",
-                                     win->show_sidebar?"1":"0", NULL);
-        bonobo_ui_component_set_prop(uic, "/commands/SettingsShowStatusbar", "state",
-                                     win->show_statusbar?"1":"0", NULL);
+        sync_settings_menu_items(win);
+
         if(!win->show_toolbar)
                 bonobo_ui_component_set_prop(uic, "/Toolbar", "hidden", "1", NULL);
         if(!win->show_menus)
@@ -943,6 +1093,26 @@ ggv_window_new(void)
                 win->pb = pb;
         }
 
+        /* merge our items in the control's popup menu */
+        popup_uic = bonobo_control_frame_get_popup_component(win->ctlframe, NULL);
+        if(popup_uic == NULL) {
+                g_warning("Control does not have a popup component.");
+        }
+        else {
+                bonobo_ui_util_set_ui(popup_uic, NULL, "ggv-ui.xml",
+                                      "GGV", NULL);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowMenus",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowSidebar",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowToolbar",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowStatusbar",
+                                                 listener_SettingsShow, win);
+                win->popup_uic = popup_uic;
+                sync_settings_popup_items(win);
+        }
+
         set_file_items_sensitivity(win, FALSE);
 
 	/* set default geometry */
@@ -1027,7 +1197,7 @@ void
 ggv_window_apply_prefs(GgvWindow *win)
 {
         if(win) {
-                /* TODO: set control properties */
+                sync_toolbar_style(win);
         }
         else {
                 GList *node = window_list;
