@@ -21,20 +21,6 @@
 #include <math.h>
 #include <ctype.h>
 
-#if 0
-#include "gtkscrollpane.h"
-#include "gtkchecklist.h"
-#include "prefs.h"
-#include "stock/foot.xpm"
-#include "stock/foot_white.xpm"
-#include "gsmessage.h"
-#include "ggvutils.h"
-#include "ggvwindow.h"
-#include "callbacks.h"
-#include "ps.h"
-#include "gsdefaults.h"
-#endif
-
 #include <Ggv.h>
 
 #include "gtkgs.h"
@@ -59,7 +45,7 @@ struct _GgvWindow
         Bonobo_PropertyBag pb;
 
         BonoboControlFrame *ctlframe;
-        BonoboUIComponent *uic;
+        BonoboUIComponent *uic, *popup_uic;
 
         GtkWidget *hbox, *vbox;
         GtkWidget *sidebar, *statusbar;
@@ -77,6 +63,8 @@ struct _GgvWindow
         gdouble prev_x, prev_y;
         gfloat xcoord, ycoord;
 
+        gchar **uris_to_open;
+
         gint current_page;
 
         /* these are values from ggv control: we keep a copy here that
