@@ -41,11 +41,6 @@ void ggv_set_tooltip(GtkWidget* w, const gchar* tip);
 gfloat ggv_compute_zoom(gint zoom_spec);
 gint ggv_compute_spec(gfloat zoom);
 
-#define STOCK_TOGGLE_ALL "GGV_toggle_all"
-#define STOCK_TOGGLE_EVEN "GGV_toggle_even"
-#define STOCK_TOGGLE_ODD "GGV_toggle_odd"
-#define STOCK_CLEAR_ALL "GGV_clear_all"
-
 void ggv_stock_init (void);
 
 void ggv_raise_and_focus_widget(GtkWidget *widget);
